// destination.gs
//
// Zielanzeige unter Verwendung von Name-Effekten
// Es gibt 3 Arten von Anzeigen, die auch alle gleichzeitig im Fahrzeug mglich sind. 
// 1. zur Anzeige des Zieles
// 2. zu Anzeige Linie oder Zugart/nummer
// 3. zur gemeinsamen Anzeige von 1. und 2.
//
// (C) p-dehnert
// 20200626

include "br143.gs"	// alternativ locomotive.gs, vehicle.gs oder eigenes Script

class Destination isclass BR143	// alternativ Locomotive, Vehicle oder eigene Klasse
{
    string destination, defaultdest, line, defaultline, linesigns, destsigns, combisigns;
    bool trimline = true;	// true: Nummer am Ende der Linie/Zugnummer entfernen, ausser bei S-Bahn (z.B. S11) oder nur Nummer (z.B. Strassenbahn)
    int MAXDEST = 20;	// maximale Lnge der Zielanzeige
    int MAXLINE = 6;	// maximale Lnge der Anzeige der Linie/Zugnummer
    string TRENNUNG = " ";	// Trennung von Linie und Ziel in der Kombianzeige
    

    bool isDigit(int char)
    {
	return ((char >= '0') and (char <= '9'));
    }

    // Eventuell Ziffern am Stringende entfernen
    string TrimLine(string pline)
    {
	int i;
	string retval;
	retval.copy(pline);
	if (!trimline or (retval.size() < 2))
	    // nicht trimmen oder hchstens 1 Zeichen
	    return retval;
	if (isDigit(retval[0]) or ((retval[0] == 'S') or (retval[0] == 'U')) and isDigit(retval[1]))
	    // fngt mit Ziffer an oder S gefolgt von Ziffer oder U gefolgt von Ziffer
	    return retval;
	// sonst Ziffern am Ende entfernen
	for (i = retval.size(); i > 1; i--)
	{
	    // Anfang der Ziffernfolge am Stringende suchen
	    if (!isDigit(retval[i - 1]))
		break;
	}
	retval = retval[,i];
	return retval;
	
    }

    // Change destination signs
    void ChangeDestinationSign(Message msg)
    {
	int i;
	string new_destination = "", new_line = "";

	if (msg)
	    new_destination = msg.minor;

	if (new_destination == null)
	{
	    new_line = defaultline;
	    line = new_line;
	    new_destination = defaultdest;
	}

	if (new_destination.size() > 0)
	{
	    if (new_destination[0,1] == "@")
		line = new_destination;
	    else
		destination = new_destination;
	}
	else
	{
	    line = "@";
	    destination = " ";
	}

	new_line = line[1, ];
	new_line = TrimLine(new_line);
	if (new_line.size() > MAXLINE)
	    new_line = new_line[, MAXLINE];
	else if (new_line == "")
	    new_line = " ";
	new_destination = destination;
	if (new_destination.size() > MAXDEST)
	    new_destination = new_destination[, MAXDEST];
	
	// Anzeige der Linie/Zugnummer
	if (linesigns != "")
	{
	    string[] ls = Str.Tokens(linesigns, ",");
	    for ( i = 0; i < ls.size(); i++ )
		SetFXNameText(ls[i], new_line);
	}
	
	// Anzeige des Ziels
	if (destsigns != "")
	{
	    string[] ds = Str.Tokens(destsigns, ",");
	    for ( i = 0; i < ds.size(); i++ )
		SetFXNameText(ds[i], new_destination);
	}
	
	// Anzeige der Kombination Linie+Ziel
	if (combisigns != "")
	{
	    string[] cs = Str.Tokens(combisigns, ",");
	    
	    if (new_destination == " ")
		new_destination = new_line;
	    else if (new_line != " ")
		new_destination = new_line + TRENNUNG + new_destination;
	    if (new_destination.size() > MAXDEST)
		new_destination = new_destination[, MAXDEST];
	    else if ((new_destination.size() < MAXDEST) and (new_line != " "))
		for ( i = new_destination.size(); i < MAXDEST; i++)
		    new_destination = new_destination + "  ";

	    for ( i = 0; i < cs.size(); i++ )
		SetFXNameText(cs[i], new_destination);
	}
    }

    //
    void GetDestInfo(void)
    {
	Soup cfg = me.GetAsset().GetConfigSoup();
	if (cfg != null)
	{
	    Soup ext = cfg.GetNamedSoup("extensions");
	    if (ext != null)
	    {
		Soup info = ext.GetNamedSoup("destinfo-206816");
		if (info != null)
		{
		    trimline = info.GetNamedTagAsBool("trimline", trimline);
		    MAXLINE = info.GetNamedTagAsInt("maxline", MAXLINE);
		    MAXDEST = info.GetNamedTagAsInt("maxdest", MAXDEST);
		    if (info.GetIndexForNamedTag("spacer") >= 0)
			TRENNUNG = info.GetNamedTag("spacer");
		    if (info.GetIndexForNamedTag("linesigns") >= 0)
			linesigns = info.GetNamedTag("linesigns");
		    else
			linesigns = "";
		    if (info.GetIndexForNamedTag("destsigns") >= 0)
			destsigns = info.GetNamedTag("destsigns");
		    else
			destsigns = "";
		    if (info.GetIndexForNamedTag("combisigns") >= 0)
			combisigns = info.GetNamedTag("combisigns");
		    else
			combisigns = "";
		    if (info.GetIndexForNamedTag("defaultline") >= 0)
			defaultline = info.GetNamedTag("defaultline");
		    else
			defaultline = "@";
		    if (info.GetIndexForNamedTag("defaultdest") >= 0)
			defaultdest = info.GetNamedTag("defaultdest");
		    else
			defaultdest = " ";
		}
	    }
	}
    }
    
    

    public void Init(void)
    {
	inherited();
	GetDestInfo();
	AddHandler(me,"ChangeDestinationSign", "", "ChangeDestinationSign");
	ChangeDestinationSign(null);
    }


    public Soup GetProperties(void)
    {
	Soup soup = inherited();

	soup.SetNamedTag("destination", destination);
	soup.SetNamedTag("line", line);

	return soup;
    }


    public void SetProperties(Soup soup)
    {
	inherited(soup);
	
	string new_destination = soup.GetNamedTag("line");
	PostMessage(me, "ChangeDestinationSign", new_destination, 0);

	new_destination = soup.GetNamedTag("destination");
	PostMessage(me, "ChangeDestinationSign", new_destination, 0);
    }

};